@@ -17,7 +17,6 @@ from torch_geometric.nn import (
     knn,
 )
 from torch_geometric.data import Batch
-import torch_points_kernels as tp
 
 from torch_points3d.core.base_conv.base_conv import *
 from torch_points3d.core.common_modules import *
@@ -139,10 +138,14 @@ class GlobalBaseModule(torch.nn.Module):
     def forward(self, data, **kwargs):
         batch_obj = Batch()
         x, pos, batch = data.x, data.pos, data.batch
-        x = self.nn(torch.cat([x, pos], dim=1))
+        if pos is not None:
+            x = self.nn(torch.cat([x, pos], dim=1))
+        else:
+            x = self.nn(x)
         x = self.pool(x, batch)
         batch_obj.x = x
-        batch_obj.pos = pos.new_zeros((x.size(0), 3))
+        if pos is not None:
+            batch_obj.pos = pos.new_zeros((x.size(0), 3))
         batch_obj.batch = torch.arange(x.size(0), device=batch.device)
         copy_from_to(data, batch_obj)
         return batch_obj
