@@ -1,5 +1,5 @@
 from typing import List, Optional
-from tqdm import tqdm as tq
+from tqdm.auto import tqdm as tq
 import itertools
 import numpy as np
 import math
@@ -7,7 +7,6 @@ import re
 import torch
 import random
 from torch.nn import functional as F
-from sklearn.neighbors import NearestNeighbors, KDTree
 from functools import partial
 
 from torch_geometric.nn import fps, radius, knn, voxel_grid
@@ -19,7 +18,8 @@ from torch_points3d.datasets.multiscale_data import MultiScaleData
 from torch_points3d.utils.transform_utils import SamplingStrategy
 from torch_points3d.utils.config import is_list
 from torch_points3d.utils import is_iterable
-from torch_points3d.core.data_transform.transforms import euler_angles_to_rotation_matrix
+from torch_points3d.utils.geometry import euler_angles_to_rotation_matrix
+
 
 class Random3AxisRotation(object):
     """
@@ -52,21 +52,21 @@ class Random3AxisRotation(object):
         self._degree_angles = [self._rot_x, self._rot_y, self._rot_z]
 
     def generate_random_rotation_matrix(self):
-        thetas = []
+        thetas = torch.zeros(3, dtype=torch.float)
         for axis_ind, deg_angle in enumerate(self._degree_angles):
             if deg_angle > 0:
-                rand_deg_angle = random.random() * deg_angle
+                rand_deg_angle = random.random() * 2 * deg_angle - deg_angle
                 rand_radian_angle = float(rand_deg_angle * np.pi) / 180.0
-                thetas.append(torch.tensor(rand_radian_angle))
-            else:
-                thetas.append(torch.tensor(0.0))
-        return euler_angles_to_rotation_matrix(thetas)
+                thetas[axis_ind] = rand_radian_angle
+        return euler_angles_to_rotation_matrix(thetas, random_order=True)
 
     def __call__(self, data):
         if self._apply_rotation:
-            pos = data.pos
+            pos = data.pos.float()
             M = self.generate_random_rotation_matrix()
             data.pos = pos @ M.T
+            if getattr(data, "norm", None) is not None:
+                data.norm = data.norm.float() @ M.T
         return data
 
     def __repr__(self):
@@ -75,6 +75,31 @@ class Random3AxisRotation(object):
         )
 
 
+class RandomTranslation(object):
+    """
+    random translation
+    Parameters
+    -----------
+    delta_min: list
+        min translation
+    delta_max: list
+        max translation
+    """
+
+    def __init__(self, delta_max: List = [1.0, 1.0, 1.0], delta_min: List = [-1.0, -1.0, -1.0]):
+        self.delta_max = torch.tensor(delta_max)
+        self.delta_min = torch.tensor(delta_min)
+
+    def __call__(self, data):
+        pos = data.pos
+        trans = torch.rand(3) * (self.delta_max - self.delta_min) + self.delta_min
+        data.pos = pos + trans
+        return data
+
+    def __repr__(self):
+        return "{}(delta_min={}, delta_max={})".format(self.__class__.__name__, self.delta_min, self.delta_max)
+
+
 class AddFeatsByKeys(object):
     """This transform takes a list of attributes names and if allowed, add them to x
 
@@ -199,6 +224,8 @@ class AddFeatByKey(object):
             if x is None:
                 if self._strict and data.pos.shape[0] != feat.shape[0]:
                     raise Exception("We expected to have an attribute x")
+                if feat.dim() == 1:
+                    feat = feat.unsqueeze(-1)
                 data.x = feat
             else:
                 if x.shape[0] == feat.shape[0]:
@@ -246,7 +273,7 @@ class NormalFeature(object):
     """
 
     def __call__(self, data):
-        if data.norm is None:
+        if getattr(data, "norm", None) is None:
             raise NotImplementedError("TODO: Implement normal computation")
 
         norm = data.norm
