@@ -12,7 +12,9 @@ from torch_points3d.metrics.registration_tracker import PatchRegistrationTracker
 from torch_points3d.metrics.registration_tracker import FragmentRegistrationTracker
 
 from torch_points3d.datasets.registration.base_siamese_dataset import BaseSiameseDataset
+from torch_points3d.datasets.registration.base_siamese_dataset import GeneralFragment
 from torch_points3d.datasets.registration.utils import compute_overlap_and_matches
+from torch_points3d.datasets.registration.test3dmatch import TestPair3DMatch
 
 
 
@@ -170,7 +172,7 @@ class Patch3DMatch(Base3DMatch):
         return size_dataset
 
 
-class Fragment3DMatch(Base3DMatch):
+class Fragment3DMatch(Base3DMatch, GeneralFragment):
     r"""
         Fragment extracted from :the Princeton 3DMatch dataset\n
         `"3DMatch: Learning Local Geometric Descriptors from RGB-D Reconstructions"
@@ -223,31 +225,37 @@ class Fragment3DMatch(Base3DMatch):
                 final dataset. (default: :obj:`None`)
             num_random_pt: number of point we select when we test
         """
+
     def __init__(
-        self,
-        root,
-        num_frame_per_fragment=50,
-        mode="train_small",
-        min_overlap_ratio=0.3,
-        max_overlap_ratio=1.0,
-        max_dist_overlap=0.01,
-        tsdf_voxel_size=0.02,
-        limit_size=700,
-        depth_thresh=6,
-        is_fine=True,
-        transform=None,
-        pre_transform=None,
-        pre_transform_fragment=None,
-        pre_filter=None,
-        verbose=False,
-        debug=False,
-        is_online_matching=False,
-        num_pos_pairs=1024,
+            self,
+            root,
+            num_frame_per_fragment=50,
+            mode="train_small",
+            min_overlap_ratio=0.3,
+            max_overlap_ratio=1.0,
+            max_dist_overlap=0.01,
+            tsdf_voxel_size=0.02,
+            limit_size=700,
+            depth_thresh=6,
+            is_fine=True,
+            transform=None,
+            pre_transform=None,
+            pre_transform_fragment=None,
+            pre_filter=None,
+            verbose=False,
+            debug=False,
+            is_online_matching=False,
+            num_pos_pairs=1024,
+            self_supervised=False,
+            min_size_block=0.3,
+            max_size_block=2,
+            ss_transform=None,
+            min_points=500,
+            use_fps=False
     ):
-
-
         self.is_patch = False
-        super(Fragment3DMatch, self).__init__(
+        Base3DMatch.__init__(
+            self,
             root,
             num_frame_per_fragment,
             mode,
@@ -269,39 +277,12 @@ class Fragment3DMatch(Base3DMatch):
         self.list_fragment = [f for f in os.listdir(self.path_match) if "matches" in f]
         self.is_online_matching = is_online_matching
         self.num_pos_pairs = num_pos_pairs
-
-    def get_fragment(self, idx):
-
-        match = np.load(osp.join(self.path_match, "matches{:06d}.npy".format(idx)), allow_pickle=True).item()
-        data_source = torch.load(match["path_source"]).to(torch.float)
-        data_target = torch.load(match["path_target"]).to(torch.float)
-        new_pair = torch.from_numpy(match["pair"])
-
-        if self.transform is not None:
-            data_source = self.transform(data_source)
-            data_target = self.transform(data_target)
-
-        if(hasattr(data_source, "multiscale")):
-            batch = MultiScalePair.make_pair(data_source, data_target)
-        else:
-            batch = Pair.make_pair(data_source, data_target)
-        if self.is_online_matching:
-            new_match = compute_overlap_and_matches(
-                Data(pos=data_source.pos), Data(pos=data_target.pos), self.max_dist_overlap
-            )
-            batch.pair_ind = torch.from_numpy(new_match["pair"].copy())
-        else:
-            pair = tracked_matches(data_source, data_target, new_pair)
-            batch.pair_ind = pair
-
-        num_pos_pairs = len(batch.pair_ind)
-        if self.num_pos_pairs < len(batch.pair_ind):
-            num_pos_pairs = self.num_pos_pairs
-
-        rand_ind = torch.randperm(len(batch.pair_ind))[:num_pos_pairs]
-        batch.pair_ind = batch.pair_ind[rand_ind]
-        batch.size_pair_ind = torch.tensor([num_pos_pairs])
-        return batch.contiguous()
+        self.self_supervised = self_supervised
+        self.min_size_block = min_size_block
+        self.max_size_block = max_size_block
+        self.ss_transform = ss_transform
+        self.min_points = min_points
+        self.use_fps = use_fps
 
     def get(self, idx):
         return self.get_fragment(idx)
@@ -309,12 +290,21 @@ class Fragment3DMatch(Base3DMatch):
     def __len__(self):
         return len(self.list_fragment)
 
+    def process(self):
+        super().process()
+
+    def download(self):
+        super().download()
+
 
 class General3DMatchDataset(BaseSiameseDataset):
     def __init__(self, dataset_opt):
         super().__init__(dataset_opt)
+
         pre_transform = self.pre_transform
+        ss_transform = getattr(self, "ss_transform", None)
         train_transform = self.train_transform
+        val_transform = self.val_transform
         test_transform = self.test_transform
         pre_filter = self.pre_filter
         test_pre_filter = getattr(self, "test_pre_filter", None)
@@ -335,10 +325,10 @@ class General3DMatchDataset(BaseSiameseDataset):
                 transform=train_transform,
                 num_random_pt=dataset_opt.num_random_pt,
                 is_offline=dataset_opt.is_offline,
-                pre_filter=pre_filter,
+                pre_filter=pre_filter
             )
 
-            self.test_dataset = Patch3DMatch(
+            self.val_dataset = Patch3DMatch(
                 root=self._data_path,
                 mode="val",
                 radius_patch=dataset_opt.radius_patch,
@@ -349,7 +339,7 @@ class General3DMatchDataset(BaseSiameseDataset):
                 limit_size=dataset_opt.limit_size,
                 depth_thresh=dataset_opt.depth_thresh,
                 pre_transform=pre_transform,
-                transform=test_transform,
+                transform=val_transform,
                 num_random_pt=dataset_opt.num_random_pt,
                 is_offline=dataset_opt.is_offline,
                 pre_filter=test_pre_filter,
@@ -369,9 +359,16 @@ class General3DMatchDataset(BaseSiameseDataset):
                 transform=train_transform,
                 pre_filter=pre_filter,
                 is_online_matching=dataset_opt.is_online_matching,
-                num_pos_pairs=dataset_opt.num_pos_pairs)
+                num_pos_pairs=dataset_opt.num_pos_pairs,
+                self_supervised=dataset_opt.self_supervised,
+                min_size_block=dataset_opt.min_size_block,
+                max_size_block=dataset_opt.max_size_block,
+                ss_transform=ss_transform,
+                min_points=dataset_opt.min_points,
+                use_fps=dataset_opt.use_fps
+            )
 
-            self.test_dataset = Fragment3DMatch(
+            self.val_dataset = Fragment3DMatch(
                 root=self._data_path,
                 mode="val",
                 num_frame_per_fragment=dataset_opt.num_frame_per_fragment,
@@ -381,22 +378,15 @@ class General3DMatchDataset(BaseSiameseDataset):
                 limit_size=dataset_opt.limit_size,
                 depth_thresh=dataset_opt.depth_thresh,
                 pre_transform=pre_transform,
-                transform=test_transform,
+                transform=val_transform,
                 is_online_matching=False,
                 num_pos_pairs=dataset_opt.num_pos_pairs,
+                self_supervised=False,
+            )
+            self.test_dataset = TestPair3DMatch(
+                root=self._data_path,
+                pre_transform=pre_transform,
+                transform=test_transform,
+                num_pos_pairs=50,
+                max_dist_overlap=dataset_opt.max_dist_overlap
             )
-
-    def get_tracker(self, wandb_log: bool, tensorboard_log: bool):
-        """
-        Factory method for the tracker
-
-        Arguments:
-            wandb_log - Log using weight and biases
-            tensorboard_log - Log using tensorboard
-        Returns:
-            [BaseTracker] -- tracker
-        """
-        if self.is_patch:
-            return PatchRegistrationTracker(self, wandb_log=wandb_log, use_tensorboard=tensorboard_log)
-        else:
-            return FragmentRegistrationTracker(self, wandb_log=wandb_log, use_tensorboard=tensorboard_log)
