@@ -2,7 +2,6 @@ import os
 import os.path as osp
 import shutil
 import json
-from tqdm import tqdm as tq
 import torch
 from glob import glob
 import sys
@@ -13,7 +12,6 @@ from plyfile import PlyData, PlyElement
 from torch_geometric.data import Data, InMemoryDataset, download_url, extract_zip
 import torch_geometric.transforms as T
 import multiprocessing
-from tqdm import tqdm
 import pandas as pd
 
 import tempfile
@@ -21,6 +19,7 @@ import urllib
 from urllib.request import urlopen
 
 from torch_points3d.datasets.base_dataset import BaseDataset
+import torch_points3d.core.data_transform as cT
 from . import IGNORE_LABEL
 
 log = logging.getLogger(__name__)
@@ -135,7 +134,9 @@ SCANNET_COLOR_MAP = {
     40: (100.0, 85.0, 144.0),
 }
 
-SPLITS = ["train", "val"]
+SPLITS = ["train", "val", "test"]
+
+MAX_NUM_POINTS = 1200000
 
 
 def get_release_scans(release_file):
@@ -176,11 +177,12 @@ def download_file(url, out_file):
         # urllib.urlretrieve(url, out_file_tmp)
         os.rename(out_file_tmp, out_file)
     else:
-        log.warning("WARNING Skipping download of existing file " + out_file)
+        pass
+        # log.warning("WARNING Skipping download of existing file " + out_file)
 
 
 def download_scan(scan_id, out_dir, file_types, use_v1_sens):
-    log.info("Downloading ScanNet " + RELEASE_NAME + " scan " + scan_id + " ...")
+    # log.info("Downloading ScanNet " + RELEASE_NAME + " scan " + scan_id + " ...")
     if not os.path.isdir(out_dir):
         os.makedirs(out_dir)
     for ft in file_types:
@@ -192,7 +194,7 @@ def download_scan(scan_id, out_dir, file_types, use_v1_sens):
         )
         out_file = out_dir + "/" + scan_id + ft
         download_file(url, out_file)
-    log.info("Downloaded scan " + scan_id)
+    # log.info("Downloaded scan " + scan_id)
 
 
 def download_label_map(out_dir):
@@ -238,8 +240,7 @@ def read_label_mapping(filename, label_from="raw_category", label_to="nyu40id"):
 
 
 def read_mesh_vertices(filename):
-    """ read XYZ for each vertex.
-    """
+    """read XYZ for each vertex."""
     assert os.path.isfile(filename)
     with open(filename, "rb") as f:
         plydata = PlyData.read(f)
@@ -252,7 +253,7 @@ def read_mesh_vertices(filename):
 
 
 def read_mesh_vertices_rgb(filename):
-    """ read XYZ RGB for each vertex.
+    """read XYZ RGB for each vertex.
     Note: RGB values are in 0-255
     """
     assert os.path.isfile(filename)
@@ -304,7 +305,7 @@ def read_segmentation(filename):
 
 
 def export(mesh_file, agg_file, seg_file, meta_file, label_map_file, output_file=None):
-    """ points are XYZ RGB (RGB in 0-255),
+    """points are XYZ RGB (RGB in 0-255),
     semantic label as nyu40 ids,
     instance label as 1-#instance,
     box as (cx,cy,cz,dx,dy,dz,semantic_label)
@@ -360,7 +361,15 @@ def export(mesh_file, agg_file, seg_file, meta_file, label_map_file, output_file
         ymax = np.max(obj_pc[:, 1])
         zmax = np.max(obj_pc[:, 2])
         bbox = np.array(
-            [(xmin + xmax) / 2, (ymin + ymax) / 2, (zmin + zmax) / 2, xmax - xmin, ymax - ymin, zmax - zmin, label_id]
+            [
+                (xmin + xmax) / 2.0,
+                (ymin + ymax) / 2.0,
+                (zmin + zmax) / 2.0,
+                xmax - xmin,
+                ymax - ymin,
+                zmax - zmin,
+                label_id,
+            ]
         )
         # NOTE: this assumes obj_id is in 1,2,3,.,,,.NUM_INSTANCES
         instance_bboxes[obj_id - 1, :] = bbox
@@ -382,7 +391,7 @@ def export(mesh_file, agg_file, seg_file, meta_file, label_map_file, output_file
 
 
 class Scannet(InMemoryDataset):
-    """ Scannet dataset, you will have to agree to terms and conditions by hitting enter
+    """Scannet dataset, you will have to agree to terms and conditions by hitting enter
     so that it downloads the dataset.
 
     http://www.scan-net.org/
@@ -392,16 +401,16 @@ class Scannet(InMemoryDataset):
     root : str
         Path to the data
     split : str, optional
-        Split used
+        Split used (train, val or test)
     transform (callable, optional):
         A function/transform that takes in an :obj:`torch_geometric.data.Data` object and returns a transformed
         version. The data object will be transformed before every access.
-    pre_transform (callable, optional): 
-        A function/transform that takes in an :obj:`torch_geometric.data.Data` object and returns a 
+    pre_transform (callable, optional):
+        A function/transform that takes in an :obj:`torch_geometric.data.Data` object and returns a
         transformed version. The data object will be transformed before being saved to disk.
-    pre_filter (callable, optional): 
+    pre_filter (callable, optional):
         A function that takes in an :obj:`torch_geometric.data.Data` object and returns a boolean
-        value, indicating whether the data object should be included in the final dataset. 
+        value, indicating whether the data object should be included in the final dataset.
     version : str, optional
         version of scannet, by default "v2"
     use_instance_labels : bool, optional
@@ -429,7 +438,7 @@ class Scannet(InMemoryDataset):
     def __init__(
         self,
         root,
-        split="trainval",
+        split="train",
         transform=None,
         pre_transform=None,
         pre_filter=None,
@@ -438,11 +447,13 @@ class Scannet(InMemoryDataset):
         use_instance_bboxes=False,
         donotcare_class_ids=[],
         max_num_point=None,
-        use_multiprocessing=False,
         process_workers=4,
         types=[".txt", "_vh_clean_2.ply", "_vh_clean_2.0.010000.segs.json", ".aggregation.json"],
         normalize_rgb=True,
+        is_test=False,
     ):
+
+        assert self.SPLITS == ["train", "val", "test"]
         if not isinstance(donotcare_class_ids, list):
             raise Exception("donotcare_class_ids should be list with indices of class to ignore")
         self.donotcare_class_ids = donotcare_class_ids
@@ -454,30 +465,90 @@ class Scannet(InMemoryDataset):
         self.max_num_point = max_num_point
         self.use_instance_labels = use_instance_labels
         self.use_instance_bboxes = use_instance_bboxes
-        self.use_multiprocessing = use_multiprocessing
+        self.use_multiprocessing = process_workers > 1
         self.process_workers = process_workers
         self.types = types
         self.normalize_rgb = normalize_rgb
-
-        super(Scannet, self).__init__(root, transform, pre_transform, pre_filter)
+        self.is_test = is_test
+        super().__init__(root, transform, pre_transform, pre_filter)
         if split == "train":
             path = self.processed_paths[0]
         elif split == "val":
             path = self.processed_paths[1]
-        elif split == "trainval":
+        elif split == "test":
             path = self.processed_paths[2]
         else:
-            raise ValueError((f"Split {split} found, but expected either " "train, val, trainval or test"))
+            raise ValueError((f"Split {split} found, but expected either " "train, val, or test"))
 
         self.data, self.slices = torch.load(path)
 
+        if split != "test":
+            if not use_instance_bboxes:
+                delattr(self.data, "instance_bboxes")
+            if not use_instance_labels:
+                delattr(self.data, "instance_labels")
+            self.data.y = self._remap_labels(self.data.y)
+            self.has_labels = True
+        else:
+            self.has_labels = False
+
+        self.read_from_metadata()
+
+    def get_raw_data(self, id_scan, remap_labels=True) -> Data:
+        """Grabs the raw data associated with a scan index
+
+        Parameters
+        ----------
+        id_scan : int or torch.Tensor
+            id of the scan
+        remap_labels : bool, optional
+            If True then labels are mapped to the range [IGNORE_LABELS:number of labels].
+            If using this method to compare ground truth against prediction then set remap_labels to True
+        """
+        stage = self.name
+        if torch.is_tensor(id_scan):
+            id_scan = int(id_scan.item())
+        assert stage in self.SPLITS
+        mapping_idx_to_scan_names = getattr(self, "MAPPING_IDX_TO_SCAN_{}_NAMES".format(stage.upper()))
+        scan_name = mapping_idx_to_scan_names[id_scan]
+        path_to_raw_scan = os.path.join(
+            self.processed_raw_paths[self.SPLITS.index(stage.lower())], "{}.pt".format(scan_name)
+        )
+        data = torch.load(path_to_raw_scan)
+        data.scan_name = scan_name
+        data.path_to_raw_scan = path_to_raw_scan
+        if self.has_labels and remap_labels:
+            data.y = self._remap_labels(data.y)
+        return data
+
     @property
     def raw_file_names(self):
         return ["metadata", "scans", "scannetv2-labels.combined.tsv"]
 
     @property
     def processed_file_names(self):
-        return ["{}.pt".format(s,) for s in Scannet.SPLITS]
+        return [
+            "{}.pt".format(
+                s,
+            )
+            for s in Scannet.SPLITS
+        ]
+
+    @property
+    def processed_raw_paths(self):
+        processed_raw_paths = [os.path.join(self.processed_dir, "raw_{}".format(s)) for s in Scannet.SPLITS]
+        for p in processed_raw_paths:
+            if not os.path.exists(p):
+                os.makedirs(p)
+        return processed_raw_paths
+
+    @property
+    def path_to_submission(self):
+        root = os.getcwd()
+        path_to_submission = os.path.join(root, "submission_labels")
+        if not os.path.exists(path_to_submission):
+            os.makedirs(path_to_submission)
+        return path_to_submission
 
     @property
     def num_classes(self):
@@ -486,9 +557,13 @@ class Scannet(InMemoryDataset):
     def download_scans(self):
         release_file = BASE_URL + RELEASE + ".txt"
         release_scans = get_release_scans(release_file)
+        # release_scans = ["scene0191_00","scene0191_01", "scene0568_00", "scene0568_01"]
         file_types = FILETYPES
+        release_test_file = BASE_URL + RELEASE + "_test.txt"
+        release_test_scans = get_release_scans(release_test_file)
         file_types_test = FILETYPES_TEST
         out_dir_scans = os.path.join(self.raw_dir, "scans")
+        out_dir_test_scans = os.path.join(self.raw_dir, "scans_test")
 
         if self.types:  # download file type
             file_types = self.types
@@ -498,7 +573,7 @@ class Scannet(InMemoryDataset):
                     return
             file_types_test = []
             for file_type in file_types:
-                if file_type not in FILETYPES_TEST:
+                if file_type in FILETYPES_TEST:
                     file_types_test.append(file_type)
         download_label_map(self.raw_dir)
         log.info("WARNING: You are downloading all ScanNet " + RELEASE_NAME + " scans of type " + file_types[0])
@@ -518,8 +593,12 @@ class Scannet(InMemoryDataset):
         download_release(release_scans, out_dir_scans, file_types, use_v1_sens=True)
         if self.version == "v2":
             download_label_map(self.raw_dir)
+            download_release(release_test_scans, out_dir_test_scans, file_types_test, use_v1_sens=True)
+            # download_file(os.path.join(BASE_URL, RELEASE_TASKS, TEST_FRAMES_FILE[0]), os.path.join(out_dir_tasks, TEST_FRAMES_FILE[0]))
 
     def download(self):
+        if self.is_test:
+            return
         log.info(
             "By pressing any key to continue you confirm that you have agreed to the ScanNet terms of use as described at:"
         )
@@ -534,6 +613,18 @@ class Scannet(InMemoryDataset):
         for url in self.URLS_METADATA:
             _ = download_url(url, metadata_path)
 
+    @staticmethod
+    def read_one_test_scan(scannet_dir, scan_name, normalize_rgb):
+        mesh_file = osp.join(scannet_dir, scan_name, scan_name + "_vh_clean_2.ply")
+        mesh_vertices = read_mesh_vertices_rgb(mesh_file)
+
+        data = {}
+        data["pos"] = torch.from_numpy(mesh_vertices[:, :3])
+        data["rgb"] = torch.from_numpy(mesh_vertices[:, 3:])
+        if normalize_rgb:
+            data["rgb"] /= 255.0
+        return Data(**data)
+
     @staticmethod
     def read_one_scan(
         scannet_dir,
@@ -542,10 +633,7 @@ class Scannet(InMemoryDataset):
         donotcare_class_ids,
         max_num_point,
         obj_class_ids,
-        mapping_func,
         normalize_rgb,
-        use_instance_labels=True,
-        use_instance_bboxes=True,
     ):
         mesh_file = osp.join(scannet_dir, scan_name, scan_name + "_vh_clean_2.ply")
         agg_file = osp.join(scannet_dir, scan_name, scan_name + ".aggregation.json")
@@ -575,9 +663,6 @@ class Scannet(InMemoryDataset):
                 semantic_labels = semantic_labels[choices]
                 instance_labels = instance_labels[choices]
 
-        # Remap labels to [0-(len(valid_labels))]
-        semantic_labels = mapping_func(semantic_labels)
-
         # Build data container
         data = {}
         data["pos"] = torch.from_numpy(mesh_vertices[:, :3])
@@ -586,12 +671,8 @@ class Scannet(InMemoryDataset):
             data["rgb"] /= 255.0
         data["y"] = torch.from_numpy(semantic_labels)
         data["x"] = None
-
-        if use_instance_labels:
-            data["iy"] = torch.from_numpy(instance_labels)
-
-        if use_instance_bboxes:
-            data["bbox"] = torch.from_numpy(instance_bboxes)
+        data["instance_labels"] = torch.from_numpy(instance_labels)
+        data["instance_bboxes"] = torch.from_numpy(instance_bboxes)
 
         return Data(**data)
 
@@ -599,91 +680,117 @@ class Scannet(InMemoryDataset):
         metadata_path = osp.join(self.raw_dir, "metadata")
         self.label_map_file = osp.join(metadata_path, LABEL_MAP_FILE)
         split_files = ["scannetv2_{}.txt".format(s) for s in Scannet.SPLITS]
-        self.scan_names = [[line.rstrip() for line in open(osp.join(metadata_path, sf))] for sf in split_files]
+        self.scan_names = []
+        for sf in split_files:
+            f = open(osp.join(metadata_path, sf))
+            self.scan_names.append(sorted([line.rstrip() for line in f]))
+            f.close()
+
+        for idx_split, split in enumerate(Scannet.SPLITS):
+            idx_mapping = {idx: scan_name for idx, scan_name in enumerate(self.scan_names[idx_split])}
+            setattr(self, "MAPPING_IDX_TO_SCAN_{}_NAMES".format(split.upper()), idx_mapping)
 
     @staticmethod
     def process_func(
         id_scan,
-        pre_transform,
+        total,
         scannet_dir,
         scan_name,
         label_map_file,
         donotcare_class_ids,
         max_num_point,
         obj_class_ids,
-        mapping_func,
         normalize_rgb,
-        use_instance_labels=True,
-        use_instance_bboxes=True,
+        split,
     ):
-        data = Scannet.read_one_scan(
-            scannet_dir,
-            scan_name,
-            label_map_file,
-            donotcare_class_ids,
-            max_num_point,
-            obj_class_ids,
-            mapping_func,
-            normalize_rgb,
-            use_instance_labels=use_instance_labels,
-            use_instance_bboxes=use_instance_bboxes,
-        )
-        if pre_transform:
-            data = pre_transform(data)
-        log.info("{}| scan_name: {}, data: {}".format(id_scan, scan_name, data))
-        return data
+        if split == "test":
+            data = Scannet.read_one_test_scan(scannet_dir, scan_name, normalize_rgb)
+        else:
+            data = Scannet.read_one_scan(
+                scannet_dir,
+                scan_name,
+                label_map_file,
+                donotcare_class_ids,
+                max_num_point,
+                obj_class_ids,
+                normalize_rgb,
+            )
+        log.info("{}/{}| scan_name: {}, data: {}".format(id_scan, total, scan_name, data))
+
+        data["id_scan"] = torch.tensor([id_scan])
+        return cT.SaveOriginalPosId()(data)
 
     def process(self):
+        if self.is_test:
+            return
         self.read_from_metadata()
 
-        mapping_dict = {indice: idx for idx, indice in enumerate(self.valid_class_idx)}
-        for idx in range(NUM_CLASSES):
-            if idx not in mapping_dict:
-                mapping_dict[idx] = IGNORE_LABEL
-        for idx in self.donotcare_class_ids:
-            mapping_dict[idx] = IGNORE_LABEL
-        print("MAPPING IDX: {}".format(mapping_dict))
-        mapping_func = np.vectorize(mapping_dict.get)
-
         scannet_dir = osp.join(self.raw_dir, "scans")
         for i, (scan_names, split) in enumerate(zip(self.scan_names, self.SPLITS)):
             if not os.path.exists(self.processed_paths[i]):
+                mapping_idx_to_scan_names = getattr(self, "MAPPING_IDX_TO_SCAN_{}_NAMES".format(split.upper()))
                 scannet_dir = osp.join(self.raw_dir, "scans" if split in ["train", "val"] else "scans_test")
                 total = len(scan_names)
                 args = [
                     (
-                        "{}/{}".format(id, total),
-                        self.pre_transform,
+                        id,
+                        total,
                         scannet_dir,
                         scan_name,
                         self.label_map_file,
                         self.donotcare_class_ids,
                         self.max_num_point,
                         self.VALID_CLASS_IDS,
-                        mapping_func,
                         self.normalize_rgb,
-                        self.use_instance_labels,
-                        self.use_instance_bboxes,
+                        split,
                     )
                     for id, scan_name in enumerate(scan_names)
                 ]
                 if self.use_multiprocessing:
-                    with multiprocessing.Pool(processes=self.process_workers) as pool:
+                    with multiprocessing.get_context("spawn").Pool(processes=self.process_workers) as pool:
                         datas = pool.starmap(Scannet.process_func, args)
                 else:
                     datas = []
                     for arg in args:
                         data = Scannet.process_func(*arg)
                         datas.append(data)
+
+                for data in datas:
+                    id_scan = int(data.id_scan.item())
+                    scan_name = mapping_idx_to_scan_names[id_scan]
+                    path_to_raw_scan = os.path.join(self.processed_raw_paths[i], "{}.pt".format(scan_name))
+                    torch.save(data, path_to_raw_scan)
+
+                if self.pre_transform:
+                    datas = [self.pre_transform(data) for data in datas]
+
                 log.info("SAVING TO {}".format(self.processed_paths[i]))
                 torch.save(self.collate(datas), self.processed_paths[i])
 
+    def _remap_labels(self, semantic_label):
+        """Remaps labels to [0 ; num_labels -1]. Can be overriden."""
+        new_labels = semantic_label.clone()
+        mapping_dict = {indice: idx for idx, indice in enumerate(self.valid_class_idx)}
+        for idx in range(NUM_CLASSES):
+            if idx not in mapping_dict:
+                mapping_dict[idx] = IGNORE_LABEL
+        for idx in self.donotcare_class_ids:
+            mapping_dict[idx] = IGNORE_LABEL
+        for source, target in mapping_dict.items():
+            mask = semantic_label == source
+            new_labels[mask] = target
+
+        broken_labels = new_labels >= len(self.valid_class_idx)
+        new_labels[broken_labels] = IGNORE_LABEL
+
+        return new_labels
+
     def __repr__(self):
         return "{}({})".format(self.__class__.__name__, len(self))
 
 
 class ScannetDataset(BaseDataset):
-    """ Wrapper around Scannet that creates train and test datasets.
+    """Wrapper around Scannet that creates train and test datasets.
 
     Parameters
     ----------
@@ -701,13 +808,17 @@ class ScannetDataset(BaseDataset):
             - val_transforms (optional)
     """
 
+    SPLITS = SPLITS
+
     def __init__(self, dataset_opt):
         super().__init__(dataset_opt)
 
         use_instance_labels: bool = dataset_opt.use_instance_labels
         use_instance_bboxes: bool = dataset_opt.use_instance_bboxes
         donotcare_class_ids: [] = dataset_opt.donotcare_class_ids if dataset_opt.donotcare_class_ids else []
-        max_num_point: int = dataset_opt.max_num_point if dataset_opt.max_num_point != "None" else None
+        max_num_point: int = dataset_opt.max_num_point if dataset_opt.max_num_point is not None else None
+        process_workers: int = dataset_opt.process_workers if dataset_opt.process_workers else 0
+        is_test: bool = dataset_opt.is_test if dataset_opt.is_test is not None else False
 
         self.train_dataset = Scannet(
             self._data_path,
@@ -719,6 +830,8 @@ class ScannetDataset(BaseDataset):
             use_instance_bboxes=use_instance_bboxes,
             donotcare_class_ids=donotcare_class_ids,
             max_num_point=max_num_point,
+            process_workers=process_workers,
+            is_test=is_test,
         )
 
         self.val_dataset = Scannet(
@@ -731,8 +844,28 @@ class ScannetDataset(BaseDataset):
             use_instance_bboxes=use_instance_bboxes,
             donotcare_class_ids=donotcare_class_ids,
             max_num_point=max_num_point,
+            process_workers=process_workers,
+            is_test=is_test,
         )
 
+        self.test_dataset = Scannet(
+            self._data_path,
+            split="test",
+            transform=self.val_transform,
+            pre_transform=self.pre_transform,
+            version=dataset_opt.version,
+            use_instance_labels=use_instance_labels,
+            use_instance_bboxes=use_instance_bboxes,
+            donotcare_class_ids=donotcare_class_ids,
+            max_num_point=max_num_point,
+            process_workers=process_workers,
+            is_test=is_test,
+        )
+
+    @property
+    def path_to_submission(self):
+        return self.train_dataset.path_to_submission
+
     def get_tracker(self, wandb_log: bool, tensorboard_log: bool):
         """Factory method for the tracker
 
@@ -742,8 +875,8 @@ class ScannetDataset(BaseDataset):
         Returns:
             [BaseTracker] -- tracker
         """
-        from torch_points3d.metrics.segmentation_tracker import SegmentationTracker
+        from torch_points3d.metrics.scannet_segmentation_tracker import ScannetSegmentationTracker
 
-        return SegmentationTracker(
+        return ScannetSegmentationTracker(
             self, wandb_log=wandb_log, use_tensorboard=tensorboard_log, ignore_label=IGNORE_LABEL
         )
