@@ -1,7 +1,7 @@
 import os
 import torchnet as tnt
 import torch
-from typing import Dict
+from typing import Dict, Any
 import wandb
 from torch.utils.tensorboard import SummaryWriter
 import logging
@@ -23,6 +23,7 @@ class BaseTracker:
         self._tensorboard_dir = os.path.join(os.getcwd(), "tensorboard")
         self._n_iter = 0
         self._finalised = False
+        self._conv_type = None
 
         if self._use_tensorboard:
             log.info(
@@ -35,7 +36,7 @@ class BaseTracker:
         self._loss_meters = {}
         self._finalised = False
 
-    def get_metrics(self, verbose=False) -> Dict[str, float]:
+    def get_metrics(self, verbose=False) -> Dict[str, Any]:
         metrics = {}
         for key, loss_meter in self._loss_meters.items():
             value = meter_value(loss_meter, dim=0)
@@ -113,3 +114,11 @@ class BaseTracker:
         for key, value in metrics.items():
             log.info("    {} = {}".format(key, value))
         log.info("".join(["=" for i in range(50)]))
+
+    @staticmethod
+    def _dict_to_str(dictionnary):
+        string = "{"
+        for key, value in dictionnary.items():
+            string += "%s: %.2f," % (str(key), value)
+        string += "}"
+        return string
