@@ -1,4 +1,4 @@
-from typing import Dict, Any, List, Optional, Tuple
+from typing import Dict, Any, List, Optional, Tuple, Union
 import os
 import torch
 import logging
@@ -32,6 +32,7 @@ class Checkpoint:
         self.stats: Dict[str, List[Any]] = {"train": [], "test": [], "val": []}
         self.optimizer: Optional[Tuple[str, Any]] = None
         self.schedulers: Dict[str, Any] = {}
+        self.dataset_properties: DictConfig = DictConfig({})
 
     def save_objects(self, models_to_save: Dict[str, Any], stage, current_stat, optimizer, schedulers, **kwargs):
         """ Saves checkpoint with updated mdoels for the given stage
@@ -46,17 +47,21 @@ class Checkpoint:
         for key, value in self.__dict__.items():
             if not key.startswith("_"):
                 to_save[key] = value
-        torch.save(to_save, self._check_path)
+        torch.save(to_save, self.path)
+
+    @property
+    def path(self):
+        return self._check_path
 
     @staticmethod
-    def load(checkpoint_dir: str, checkpoint_name: str, run_config: DictConfig, strict=False):
+    def load(checkpoint_dir: str, checkpoint_name: str, run_config: DictConfig, strict=False, resume=True):
         """ Creates a new checkpoint object in the current working directory by loading the
         checkpoint located at [checkpointdir]/[checkpoint_name].pt
         """
         checkpoint_file = os.path.join(checkpoint_dir, checkpoint_name) + ".pt"
         if not os.path.exists(checkpoint_file):
             ckp = Checkpoint(checkpoint_file)
-            if strict:
+            if strict or resume:
                 available_checkpoints = glob.glob(os.path.join(checkpoint_dir, "*.pt"))
                 message = "The provided path {} didn't contain the checkpoint_file {}".format(
                     checkpoint_dir, checkpoint_name + ".pt"
@@ -68,9 +73,10 @@ class Checkpoint:
             return ckp
         else:
             chkp_name = os.path.basename(checkpoint_file)
-            shutil.copyfile(
-                checkpoint_file, chkp_name
-            )  # Copy checkpoint to new run directory to make sure we don't override
+            if resume:
+                shutil.copyfile(
+                    checkpoint_file, chkp_name
+                )  # Copy checkpoint to new run directory to make sure we don't override
             ckp = Checkpoint(chkp_name)
             log.info("Loading checkpoint from {}".format(checkpoint_file))
             objects = torch.load(checkpoint_file, map_location="cpu")
@@ -83,7 +89,7 @@ class Checkpoint:
     def is_empty(self):
         return not self._filled
 
-    def get_optimizer(self, model):
+    def get_optimizer(self, model, load_state=True):
         if not self.is_empty:
             try:
                 optimizer_config = self.optimizer
@@ -94,12 +100,13 @@ class Checkpoint:
                 except:
                     pass
                 optimizer = optimizer_cls(model.parameters(), **optimizer_params)
-                optimizer.load_state_dict(optimizer_config[1])
+                if load_state:
+                    optimizer.load_state_dict(optimizer_config[1])
                 return optimizer
             except:
                 raise KeyError("The checkpoint doesn t contain an optimizer")
 
-    def get_schedulers(self, model):
+    def get_schedulers(self, model, load_state=True):
         if not self.is_empty:
             try:
                 schedulers_out = {}
@@ -108,11 +115,13 @@ class Checkpoint:
                     if scheduler_type == "lr_scheduler":
                         optimizer = model.optimizer
                         scheduler = instantiate_scheduler(optimizer, scheduler_opt)
-                        scheduler.load_state_dict(scheduler_state)
+                        if load_state:
+                            scheduler.load_state_dict(scheduler_state)
                         schedulers_out["lr_scheduler"] = scheduler
                     elif scheduler_type == "bn_scheduler":
                         scheduler = instantiate_bn_scheduler(model, scheduler_opt)
-                        scheduler.load_state_dict(scheduler_state)
+                        if load_state:
+                            scheduler.load_state_dict(scheduler_state)
                         schedulers_out["bn_scheduler"] = scheduler
                     else:
                         raise NotImplementedError
@@ -162,7 +171,9 @@ class ModelCheckpoint(object):
         resume=False,
         strict=False,
     ):
-        self._checkpoint = Checkpoint.load(load_dir, check_name, copy.deepcopy(run_config), strict=(strict or resume))
+        self._checkpoint = Checkpoint.load(
+            load_dir, check_name, copy.deepcopy(run_config), strict=strict, resume=resume
+        )
         self._resume = resume
         self._selection_stage = selection_stage
 
@@ -170,6 +181,10 @@ class ModelCheckpoint(object):
         if not self.is_empty:
             run_config = copy.deepcopy(self._checkpoint.run_config)
             model = instantiate_model(run_config, dataset)
+            if hasattr(self._checkpoint, "model_props"):
+                for k, v in self._checkpoint.model_props.items():
+                    setattr(model, k, v)
+                delattr(self._checkpoint, "model_props")
             self._initialize_model(model, weight_name)
             return model
         else:
@@ -182,6 +197,10 @@ class ModelCheckpoint(object):
         else:
             return 1
 
+    @property
+    def run_config(self):
+        return self._checkpoint.run_config
+
     @property
     def data_config(self):
         return self._checkpoint.run_config.data
@@ -198,24 +217,38 @@ class ModelCheckpoint(object):
     def is_empty(self):
         return self._checkpoint.is_empty
 
+    @property
+    def checkpoint_path(self):
+        return self._checkpoint.path
+
+    @property
+    def dataset_properties(self) -> DictConfig:
+        return self._checkpoint.dataset_properties
+
+    @dataset_properties.setter
+    def dataset_properties(self, dataset_properties: Union[Dict[str, Any], DictConfig]):
+        if isinstance(dataset_properties, dict):
+            dataset_properties = DictConfig(dataset_properties)
+        self._checkpoint.dataset_properties = dataset_properties
+
     def get_starting_epoch(self):
         return len(self._checkpoint.stats["train"]) + 1
 
     def _initialize_model(self, model: model_interface.CheckpointInterface, weight_name):
         if not self._checkpoint.is_empty:
             state_dict = self._checkpoint.get_state_dict(weight_name)
-            model.load_state_dict(state_dict)
-            if self._resume:
-                model.optimizer = self._checkpoint.get_optimizer(model)
-                model.schedulers = self._checkpoint.get_schedulers(model)
+            model.load_state_dict(state_dict, strict=False)
+            model.optimizer = self._checkpoint.get_optimizer(model, load_state=self._resume)
+            model.schedulers = self._checkpoint.get_schedulers(model, load_state=self._resume)
 
     def find_func_from_metric_name(self, metric_name, default_metrics_func):
         for token_name, func in default_metrics_func.items():
             if token_name in metric_name:
                 return func
         raise Exception(
-            'The metric name %s doesn t have a func to measure which one is best. Example: For best_train_iou, {"iou":max}'
-            % token_name
+            'The metric name {} doesn t have a func to measure which one is best in {}. Example: For best_train_iou, {{"iou":max}}'.format(
+                metric_name, default_metrics_func
+            )
         )
 
     def save_best_models_under_current_metrics(
@@ -275,5 +308,24 @@ class ModelCheckpoint(object):
                     current_stat["best_{}".format(metric_name)] = metric_value
                     models_to_save["best_{}".format(metric_name)] = state_dict
 
+        kwargs["model_props"] = {
+            "num_epochs": model.num_epochs,  # type: ignore
+            "num_batches": model.num_batches,  # type: ignore
+            "num_samples": model.num_samples,  # type: ignore
+        }
+
         self._checkpoint.stats[stage].append(current_stat)
         self._checkpoint.save_objects(models_to_save, stage, current_stat, model.optimizer, model.schedulers, **kwargs)
+
+    def validate(self, dataset_config):
+        """ A checkpoint is considered as valid if it can recreate the model from
+        a dataset config only """
+        if dataset_config is not None:
+            for k, v in dataset_config.items():
+                self.data_config[k] = v
+        try:
+            instantiate_model(self.run_config, self.data_config)
+        except:
+            return False
+
+        return True
