@@ -283,7 +283,7 @@ class UnetSkipConnectionBlock(nn.Module):
             self.submodule = submodule
             self.up = upconv
 
-    def forward(self, data, **kwargs):
+    def forward(self, data, *args, **kwargs):
         if self.innermost:
             data_out = self.inner(data, **kwargs)
             data = (data_out, data)
@@ -350,6 +350,21 @@ class UnwrappedUnetBasedModel(BaseModel):
         else:
             self._init_from_compact_format(opt, model_type, dataset, modules_lib)
 
+    def _collect_sampling_ids(self, list_data):
+        def extract_matching_key(keys, start_token):
+            for key in keys:
+                if key.startswith(start_token):
+                    return key
+            return None
+
+        d = {}
+        if self.save_sampling_id:
+            for idx, data in enumerate(list_data):
+                key = extract_matching_key(data.keys, "sampling_id")
+                if key:
+                    d[key] = getattr(data, key)
+        return d
+
     def _get_from_kwargs(self, kwargs, name):
         module = kwargs[name]
         kwargs.pop(name)
@@ -380,6 +395,8 @@ class UnwrappedUnetBasedModel(BaseModel):
         self.inner_modules = nn.ModuleList()
         self.up_modules = nn.ModuleList()
 
+        self.save_sampling_id = opt.down_conv.save_sampling_id
+
         # Factory for creating up and down modules
         factory_module_cls = self._get_factory(model_type, modules_lib)
         down_conv_cls_name = opt.down_conv.module_name
@@ -470,7 +487,7 @@ class UnwrappedUnetBasedModel(BaseModel):
 
         return flattenedOpts
 
-    def forward(self, data, precomputed_down=None, precomputed_up=None):
+    def forward(self, data, precomputed_down=None, precomputed_up=None, **kwargs):
         """ This method does a forward on the Unet assuming symmetrical skip connections
 
         Parameters
@@ -492,6 +509,11 @@ class UnwrappedUnetBasedModel(BaseModel):
             stack_down.append(data)
             data = self.inner_modules[0](data)
 
+        sampling_ids = self._collect_sampling_ids(stack_down)
+
         for i in range(len(self.up_modules)):
             data = self.up_modules[i]((data, stack_down.pop()), precomputed=precomputed_up)
+
+        for key, value in sampling_ids.items():
+            setattr(data, key, value)
         return data
