@@ -1,6 +1,7 @@
 from collections import OrderedDict
 from abc import abstractmethod
-from typing import Optional, Dict, Any
+from typing import Optional, Dict, Any, List
+import os
 import torch
 from torch.optim.optimizer import Optimizer
 from torch.optim.lr_scheduler import _LRScheduler
@@ -28,6 +29,9 @@ class BaseModel(torch.nn.Module, TrackerInterface, DatasetInterface, CheckpointI
         -- <optimize_parameters>:           calculate losses, gradients, and update network weights.
     """
 
+    __REQUIRED_DATA__: List[str] = []
+    __REQUIRED_LABELS__: List[str] = []
+
     def __init__(self, opt):
         """Initialize the BaseModel class.
         Parameters:
@@ -53,9 +57,6 @@ class BaseModel(torch.nn.Module, TrackerInterface, DatasetInterface, CheckpointI
         self._num_epochs = None
         self._num_batches = 0
         self._num_samples = -1
-        self._latest_metrics = None
-        self._latest_stage = None
-        self._latest_epoch = None
         self._schedulers = {}
         self._accumulated_gradient_step = None
         self._grad_clip = -1
@@ -85,6 +86,30 @@ class BaseModel(torch.nn.Module, TrackerInterface, DatasetInterface, CheckpointI
     def optimizer(self, optimizer):
         self._optimizer = optimizer
 
+    @property
+    def num_epochs(self):
+        return self._num_epochs
+
+    @num_epochs.setter
+    def num_epochs(self, num_epochs):
+        self._num_epochs = num_epochs
+
+    @property
+    def num_batches(self):
+        return self._num_batches
+
+    @num_batches.setter
+    def num_batches(self, num_batches):
+        self._num_batches = num_batches
+
+    @property
+    def num_samples(self):
+        return self._num_samples
+
+    @num_samples.setter
+    def num_samples(self, num_samples):
+        self._num_samples = num_samples
+
     @property
     def learning_rate(self):
         for param_group in self.optimizer.param_groups:
@@ -98,6 +123,10 @@ class BaseModel(torch.nn.Module, TrackerInterface, DatasetInterface, CheckpointI
     def conv_type(self):
         return self._conv_type
 
+    @conv_type.setter
+    def conv_type(self, conv_type):
+        self._conv_type = conv_type
+
     def set_input(self, input, device):
         """Unpack input data from the dataloader and perform necessary pre-processing steps.
         Parameters:
@@ -105,6 +134,24 @@ class BaseModel(torch.nn.Module, TrackerInterface, DatasetInterface, CheckpointI
         """
         raise NotImplementedError
 
+    def load_state_dict_with_same_shape(self, weights, strict=False):
+        model_state = self.state_dict()
+        filtered_weights = {k: v for k, v in weights.items() if k in model_state and v.size() == model_state[k].size()}
+        log.info("Loading weights:" + ", ".join(filtered_weights.keys()))
+        self.load_state_dict(filtered_weights, strict=strict)
+
+    def set_pretrained_weights(self):
+        path_pretrained = getattr(self.opt, "path_pretrained", None)
+        weight_name = getattr(self.opt, "weight_name", "latest")
+
+        if path_pretrained is not None:
+            if not os.path.exists(path_pretrained):
+                log.warning("The path does not exist, it will not load any model")
+            else:
+                log.info("load pretrained weights from {}".format(path_pretrained))
+                m = torch.load(path_pretrained)["models"][weight_name]
+                self.load_state_dict_with_same_shape(m, strict=False)
+
     def get_labels(self):
         """ returns a trensor of size ``[N_points]`` where each value is the label of a point
         """
@@ -165,7 +212,7 @@ class BaseModel(torch.nn.Module, TrackerInterface, DatasetInterface, CheckpointI
         self._num_batches += 1
         self._num_samples += batch_size
 
-        self.forward()  # first call forward to calculate intermediate results
+        self.forward(epoch=epoch)  # first call forward to calculate intermediate results
         make_optimizer_step = self._manage_optimizer_zero_grad()  # Accumulate gradient if option is up
         self.backward()  # calculate gradients
 
@@ -383,11 +430,29 @@ class BaseModel(torch.nn.Module, TrackerInterface, DatasetInterface, CheckpointI
                         state[k] = v.to(*args, **kwargs)
         return self
 
-    def cpu(self):
-        return self.to(torch.device("cpu"))
+    def verify_data(self, data, forward_only=False):
+        """ Goes through the __REQUIRED_DATA__ and __REQUIRED_LABELS__ attribute of the model
+        and verifies that the passed data object contains all required members.
+        If something is missing it raises a KeyError exception.
+        """
+        missing_keys = []
+        required_attributes = self.__REQUIRED_DATA__
+        if not forward_only:
+            required_attributes += self.__REQUIRED_LABELS__
+        for attr in required_attributes:
+            if not hasattr(data, attr) or data[attr] is None:
+                missing_keys.append(attr)
+        if len(missing_keys):
+            raise KeyError(
+                "Missing attributes in your data object: {}. The model will fail to forward.".format(missing_keys)
+            )
 
-    def cuda(self):
-        return self.to(torch.device("cuda"))
+    def print_transforms(self):
+        message = ""
+        for attr in self.__dict__:
+            if "transform" in attr:
+                message += "{}{} {}= {}\n".format(COLORS.IPurple, attr, COLORS.END_NO_TOKEN, getattr(self, attr))
+        print(message)
 
 
 class BaseInternalLossModule(torch.nn.Module):
